module AwesomePrint
  module Formatters
    require 'awesome_print/formatters/object_formatter'
    require 'awesome_print/formatters/struct_formatter'
    require 'awesome_print/formatters/hash_formatter'
    require 'awesome_print/formatters/array_formatter'
    require 'awesome_print/formatters/simple_formatter'
    require 'awesome_print/formatters/method_formatter'
    require 'awesome_print/formatters/class_formatter'
    require 'awesome_print/formatters/dir_formatter'
    require 'awesome_print/formatters/file_formatter'
    require 'awesome_print/colorize'
  end
end

